/*
 * @(#)CbnBuses.java        2.0 01/25/2005
 *
 * Copyright (c) 2004 CAVS, All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted.
 *
 */
package bullybus;
import java.io.*;
import java.util.*;
/**
 * The class is a container for location information of a bus.
 *
 *
 * @author Ron Lewis
 * @version 2.0 02/01/2005
 */
public class CbnBus implements Serializable{
    /**
     * The bus id.
     */
    private String id;
    /**
     * The route this bus operates.
     */
    private String route;
    /**
     * The north south gps coordinate, aka latitude.
     */
    private float gpsN;
    /**
     * The east west gps coordinate, aka longitude.
     */ 
    private float gpsW;
    /**
     * The previous gpsN.
     */
    private float gpsN1;
    /**
     * The previous gpsW.
     */
    private float gpsW1;
    /**
     * Constructor.
     * Creates a new bus, given initial data.
     */
    public CbnBus(String id, String r,float n, float w, float n1, float w1)
    {
	this.id = id;
	this.route = r;
	this.gpsN = n;
	this.gpsW = w;
	this.gpsN1 = n1;
	this.gpsW1 = w1;
    }
    /**
     * Constructor.
     * Creates a bus with no information.
     */
    public CbnBus()
    {
	this.id = null;
	this.route = null;
	this.gpsW = 0;
	this.gpsN = 0; 
	this.gpsW1 = 0;
	this.gpsN1 = 0;
    }
    /**
     * Gets the bus id.
     * @return The bus id in a String.
     */
    public String getId(){ 
	return id; 
    }
    /**
     * Gets the route name.
     * @return The route name in a String.
     */
    public String getRoute(){ 
	return route; 
    }
    /**
     * Gets the North South gps coordinate
     * @return The North South gps coordinate.
     */
    public float getGpsN(){ 
	return gpsN; 
    }
     /**
     * Gets the East West gps coordinate
     * @return The North South gps coordinate.
     */
    public float getGpsW(){ 
	return gpsW; 
    }
    /**
     * Gets the previous North South gps coordinate
     * @return The North South gps coordinate.
     */
    public float getGpsN1(){ 
	return gpsN1; 
    }
    /**
     * Gets the previous East West gps coordinate
     * @return The North South gps coordinate.
     */
    public float getGpsW1(){ 
	return gpsW1; 
    }
    /**
     * Set all the data.
     * @param id The bus id.
     * @param route The route this bus operates.
     * @param gpsN The north south gps coordinate.
     * @param gpsW The east west gps coordinate.
     * @param gpsN1 The previous north gps coordinate.
     * @param gpsW1 the previous west gps coordinate.
     */
    public void setData(String id,String route, float gpsN, float gpsW,float gpsN1, float gpsW1){ 
	this.id = id;
	this.route = route;
	this.gpsN = gpsN;
	this.gpsW = gpsW;
	this.gpsN1 = gpsN1;
	this.gpsW1 = gpsW1;	
    }
    /**
     * Set the bus id.
     * @param id the bus id.
     */
    public void setId(String id){ 
	this.id = id; 
    }
    /**
     * Set the route.
     * @param route The route name this bus operates.
     */
    public void setRoute(String route){ 
	this.route = route; 
    }  
    /**
     * Set the route.
     * @param gpsN The gpsN coordinate.
     * @param gpsW The gpsW coordinate.
     */
    public void setGps(float gpsN, float gpsW){ 
	this.gpsN = gpsN; 
	this.gpsW = gpsW; 
    }
    /**
     * Set the route.
     * @param gpsN The gpsN coordinate.
     */
    public void setGpsN(float gpsN){ 
	this.gpsN = gpsN; 
    }
    /**
     * Set the gpsW variable.
     * @param gpsW The gpsW coordinate.
     */
    public void setGpsW(float gpsW){ 
	this.gpsW = gpsW;
    }
    /**
     * Set the gpsN1 variable.
     * @param gpsN1 The previous gpsN coordinate.
     */
    public void setGpsN1(float gpsN1){ 
	this.gpsN1 = gpsN1;
    }
    /**
     * Set the gpsW1 variable.
     * @param gpsW1 The previous gpsW coordinate.
     */
    public void setGpsW1(float gpsW1){ 
	this.gpsW1 = gpsW1;
    }
}





















