/*
 * @(#)CbnMap.java        1.0 2004/07/30
 *
 * Copyright (c) 2004 CAVS, All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted.
 *
 */
package bullybus;
import java.awt.*;
import java.awt.Canvas.*;
import java.awt.image.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.Timer;
import java.util.TimerTask;
import java.awt.Toolkit;
/** 
 * A class to display an image on a canvas and 
 * turn it into a map for the purpose of tracking
 * the movement of buses.
 *
 * @author Ron Lewis
 * @version 2.0 02/01/2005
 */
public class CbnMap extends java.awt.Canvas
{
   /**
    * The ActionListener that events are fired to.
    */
   private ActionListener actionListener;  // post action events to listeners
   /**
    * The image to draw on, a jpg of a map.
    */
   protected Image image;
   /**
    * This is the graphics buffer, helps fix flicker.
    */
   protected Image offscreen;  // Double Buffer
    /**
     * Map Corner Coordinates.
     */
   public static double nwLat = 33.4977;
    /**
     * Map Corner Coordinates.
     */
   public static double nwLon = 88.8565;
    /**
     * Map Corner Coordinates.
     */
   public static double seLat = 33.4387;
    /**
     * Map Corner Coordinates.
     */
   public static double seLon = 88.7578;
   /**
    * The blue route overlay.
    */
   protected Image blueRoute;
   /** 
    * The gray route overlay.
    */
   protected Image grayRoute;
   /**
    * The green route overlay.
    */
   protected Image greenRoute;
   /**
    * The maroon route overlay.
    */
   protected Image maroonRoute;
   /**
    * The night route overlay.
    */
   protected Image nightRoute;
   /**
    * The blue bus icon.
    */
   protected Image blueBusIcon;
   /**
    * The green bus icon.
    */
   protected Image greenBusIcon; 
   /**
    * The gray bus icon.
    */
   protected Image grayBusIcon;
   /**
    * The maroon bus icon.
    */
   protected Image maroonBusIcon; 
   /**
    * The maroon bus icon.
    */
   protected Image nightBusIcon;
   /**
    * The image layer that contains the streets.
    */
   protected Image street_layer;
   /**
    * The image layer that contains the roads.
    */
   protected Image road_layer;
   /**
    * The image layer that contains an outline of the MSU Campus
    */
   protected Image msu_layer;
   /**
    * The image layer that contains an outline of the City of Starkville
    */
   protected Image starkville_layer;
   /**
    * The image layer that contains the highway data.
    */
   protected Image highway_layer;
    /**
    * The image layer that contains the water ways data.
    */   
   protected Image water_layer;
   /**
    * The image layer that contains the text data.
    */   
   protected Image text_layer;
   /**
    * The width of the image.
    */
   protected int width = -1;
   /**
    * The height of the image.
    */
   protected int height = -1;
   /**
    * The zoom percentage.
    */
    protected double zoomFactor = 1.0; // Zoom is 100% by default
   /**
    * The set of stops to paint.
    */
   protected CbnStopSet stops;
   /**
    * The bus data to paint on the map.
    */
   protected CbnBuses buses;
   /**
    * Timer to update the bus data.
    */
   public Timer timer;   
   /**
    * Timer to center and zoom the map.
    */
   public Timer cTimer;
   /**
    * If true draw the blue Route overlay.
    */
   public boolean drawBlueRoute = false;
   /**
    * If true draw the gray Route overlay.
    */
   public boolean drawGrayRoute = false;
   /**
    * If true draw the green Route overlay.
    */
   public boolean drawGreenRoute = false;
   /**
    * If true draw the maroon Route overlay.
    */
   public boolean drawMaroonRoute = false;
  /**
    * If true draw the night Route overlay.
    */
   public boolean drawNightRoute = false;
   /**
   * The Background color of the map.
   *
   */
   protected Color backG;
   /**
    * Constructs a new map from and image, a picture of a map,
    * and a string, the file name of the map. 
    * @param image the picture of the map.
    */
   public CbnMap (Image image) 
   {
       // The image is a transparent gif that is the
       // size we want the map drawn.
      this.image = image;
      // Set the size of the canvas to the image size.
      setSize();
      //Set the background to white.
      setBackground(Color.white);

      // We need to load all the images from the jar file.
      greenBusIcon = loadImage("/map/greenBusIcon.gif");
      blueBusIcon = loadImage("/map/blueBusIcon.gif");
      grayBusIcon = loadImage("/map/grayBusIcon.gif");
      maroonBusIcon = loadImage("/map/maroonBusIcon.gif");
      nightBusIcon = loadImage("/map/nightBusIcon.gif");
      blueRoute = loadImage("/map/blue_route.gif");
      greenRoute = loadImage("/map/green_route.gif");
      grayRoute = loadImage("/map/gray_route.gif");
      maroonRoute = loadImage("/map/maroon_route.gif");
      nightRoute = loadImage("/map/night_route.gif");
      street_layer = loadImage("/map/street_layer.gif");
      highway_layer = loadImage("/map/highway_layer.gif");
      text_layer = loadImage("/map/text_layer.gif");
      msu_layer = loadImage("/map/msu_layer.gif");
      starkville_layer = loadImage("/map/starkville_layer.gif");
      water_layer = loadImage("/map/water_layer.gif");
      road_layer = loadImage("/map/road_layer.gif");
      backG = new Color(224,228,201);

      // Setup the event timers.  
      timer = new Timer();
      cTimer = new Timer();
      //This time fires the getBus action.
      timer.schedule(new GetBusesTask(),0, 1*1000);
      //This one fires the center map action.
      cTimer.schedule(new CenterTask(),0,2*1000);
   }
   /**
    * Constructs a new map from an imageProducer, a picture of a map,
    * and a string, the file name of the map. 
    * @param ip an Image Producer of the map image.
    */
   public CbnMap (ImageProducer ip) 
   {
      // The image is a transparent gif that is the
      // size we want the map drawn.
      image = createImage(ip);
      // Set the size of the canvas to the image size.
      setSize();
      //Set the background to white.
      setBackground(Color.white);

      // We need to load all the images from the jar file.
      greenBusIcon = loadImage("/map/greenBusIcon.gif");
      blueBusIcon = loadImage("/map/blueBusIcon.gif");
      grayBusIcon = loadImage("/map/grayBusIcon.gif");
      maroonBusIcon = loadImage("/map/maroonBusIcon.gif");
      nightBusIcon = loadImage("/map/nightBusIcon.gif");
      blueRoute = loadImage("/map/blue_route.gif");
      greenRoute = loadImage("/map/green_route.gif");
      grayRoute = loadImage("/map/gray_route.gif");
      maroonRoute = loadImage("/map/maroon_route.gif");
      nightRoute = loadImage("/map/night_route.gif");
      street_layer = loadImage("/map/street_layer.gif");
      highway_layer = loadImage("/map/highway_layer.gif");
      text_layer = loadImage("/map/text_layer.gif");
      msu_layer = loadImage("/map/msu_layer.gif");
      starkville_layer = loadImage("/map/starkville_layer.gif");
      water_layer = loadImage("/map/water_layer.gif");
      road_layer = loadImage("/map/road_layer.gif");
      backG = new Color(224,228,201);

      // Setup the event timers.  
      timer = new Timer();
      cTimer = new Timer();
      //This time fires the getBus action.
      timer.schedule(new GetBusesTask(),0, 1*1000);
      //This one fires the center map action.
      cTimer.schedule(new CenterTask(),0,2*1000);
   }
   /**
    * Set the size of the canvas to the size of the image.
    */
   protected synchronized void setSize () 
   {
      if (width < 0) 
         width = image.getWidth(this);
      if (height < 0) 
         height = image.getHeight(this);
      if (width > 0 && height > 0) 
         setSize((int)(width*zoomFactor),(int)(height*zoomFactor));
     
   }
   /**
    * Make sure the ImageObserver is updated properly. 
    */
    public synchronized boolean imageUpdate (Image img, int infoflags, int x, int y, int w, int h) 
    {
       if (this.width < 0 || this.height < 0) 
       {
          if ((infoflags & ImageObserver.WIDTH) != 0) 
             this.width = w;
          if ((infoflags & ImageObserver.HEIGHT) != 0) 
             this.height = h;
          if (this.width > 0 && this.height > 0) 
          {
             setSize((int)(this.width*zoomFactor) , (int)(this.height*zoomFactor));
	     Component c = this;
	     while ((c = c.getParent()) != null && ! (c instanceof Window));
	     if (c != null) 
	        ((Window)c).pack();
          }
       }
       return super.imageUpdate(img, infoflags, x, y, width, height);
    }
   /**
    * Read an image from the server and create a local copy.
    * @param file the path and filename of the image to load
    * the server name is hard coded.
    */
   public Image loadImage (String file) 
   {
      URL url = null;
      Image imageRet = null;
      try{
	 url = this.getClass().getResource(file);
	 Object o = url.getContent();
	 if (o == null || ! (o instanceof ImageProducer))
	    System.err.println(o +": no ImageProducer");
	 else 
	 {
	    // Create new Image
	    imageRet = createImage((ImageProducer)o);				

	  }

     }catch (MalformedURLException me){
        System.err.println(me);
	System.out.println(url.toString());
      }catch (IOException ie){
	  System.err.println(ie);
      }
      return imageRet;
   } // End loadImage(String)
   /**
    * Called to repaint().
    */
    public void update(Graphics g){
	paint(g);
    }
   /**
    * The main paint routine, paints the images, the buses,
    * and the stops.
    */   
   public void paint(Graphics g)
   {
      Dimension d = getSize(); // Get the proper size.
      if(offscreen == null) {  // Create the offscreen image if it is null.
	  offscreen = createImage(image.getWidth(this), image.getHeight(this));
      }
      Graphics og = offscreen.getGraphics();  // Create the off screen graphics
      if (width > 0 && height > 0 && d.width > 0 && d.height > 0) // Check the aspect ratio
      {
         float hAspect = (float)d.height/height, wAspect = (float)d.width/width;
	 int h1, w1, w ,h;
	 if (hAspect < wAspect) 
	 { 
	    h = (int)(image.getHeight(this)* zoomFactor); 
	    w = (int)(image.getWidth(this)* zoomFactor);
	    w1 = (int)(image.getWidth(this));
	    h1 = (int)(image.getHeight(this));
	 }
	 else 
	 { 
	    w = (int)(image.getHeight(this) * zoomFactor);
	    h = (int)(image.getWidth(this) * zoomFactor);
	    h1 = (int)(image.getWidth(this));
	    w1 = (int)(image.getHeight(this));
	 }
	 //Draw the map layers.
         og.setColor(backG);
         og.fillRect(0,0,h1,w1);
	 //og.drawImage(image, 0, 0,h1,w1, null);
	 og.drawImage(starkville_layer,0, 0,h1,w1, null);
         og.drawImage(msu_layer,0, 0,h1,w1, null);
         //og.drawImage(water_layer,0, 0,h1,w1, null);
	 og.drawImage(road_layer,0, 0,h1,w1, null);
         og.drawImage(street_layer,0, 0,h1,w1, null);
         og.drawImage(highway_layer,0, 0,h1,w1, null);
         og.drawImage(text_layer,0, 0,h1,w1, null);

         if(drawBlueRoute) // Draw the blue route overlay if it is turned on
	 {
	     og.drawImage(blueRoute, 0, 0,h1,w1, null);
	 }
         if(drawGrayRoute)// Draw the gray route overlay if it is turned on
	 {
	     og.drawImage(grayRoute, 0, 0,h1,w1, null);
	 }
         if(drawGreenRoute)// Draw the green route overlay if it is turned on
	 {
	     og.drawImage(greenRoute, 0, 0,h1,w1, null);
	 }
         if(drawMaroonRoute)// Draw the maroon route overlay if it is turned on
	 {
	     og.drawImage(maroonRoute, 0, 0,h1,w1, null);
	 }
	 if(drawNightRoute)
	 {
	     og.drawImage(nightRoute, 0, 0,h1,w1, null);
	 }    
	 int paintX, paintY;
   
	 for(int i = 0; i < stops.getSize(); i++) //Draw all the stops.
	 {
	    paintY = calculateY(stops.getStop(i).getGpsN());
	    paintX = calculateX(stops.getStop(i).getGpsW());
            og.setColor(Color.BLACK);
            og.fillOval(paintX-10, paintY-10, 20 , 20);
	    og.setColor(new Color(224,228,201));

	    og.fillOval(paintX-5, paintY-5, 10 , 10);
	    og.setColor(Color.lightGray);
	
	    og.setColor(Color.black);

	 }
	 int iS = 36/2; //The bus icon size;
	 try{
	     for( int j = 0; j < buses.getSize(); j++)  // Draw the buses with the proper icon
	     {
		 paintX = calculateX(buses.getBus(j).getGpsW());
		 paintY = calculateY(buses.getBus(j).getGpsN());
		 if(buses.getBus(j).getRoute().equals("blue") || buses.getBus(j).getRoute().equals("Blue"))
		 {
		     og.drawImage(blueBusIcon, paintX - iS,paintY - iS, this);
		 }
		 else if( buses.getBus(j).getRoute().equals("gray") || buses.getBus(j).getRoute().equals("Gray"))
		 {
		     og.drawImage(grayBusIcon, paintX - iS,paintY - iS, this);
		 }
		 else if( buses.getBus(j).getRoute().equals("green") || buses.getBus(j).getRoute().equals("Green"))
		 {
		     og.drawImage(greenBusIcon, paintX - iS,paintY - iS, this);
		 }
		 else if( buses.getBus(j).getRoute().equals("maroon") || buses.getBus(j).getRoute().equals("Maroon"))
		 {
		     og.drawImage(maroonBusIcon, paintX - iS,paintY - iS, this);
		 }		 
		 else if( buses.getBus(j).getRoute().equals("night") || buses.getBus(j).getRoute().equals("Night"))
		 {
		     og.drawImage(nightBusIcon, paintX - iS,paintY - iS, this);   
		 }
		 else{
		     //Do noda
		 }
	     }	
	 }catch(NullPointerException e){ }	
         
         g.drawImage(offscreen, 0, 0,h,w, this); //Blast the completely drawn image to the screen.
         og.dispose();  // Remove the offscreen graphics.  
	 System.gc();
      }			
   }
   /**
    * Returns the width of the image.
    * @return width
    */
   public int getImageWidth(){
      return this.width;
   }
   /**
    * returns the height of the image.
    * @return height
    */
   public int getImageHeight(){
      return this.height;
   }
   /**
    * Calculate map coordinates from gps coordinates.
    */
   public int calculateY(float y){

      double nY = ((0-image.getHeight(this))/(nwLat - seLat))*( y - nwLat);

      return (int)(nY*map.zoomFactor);
    }
   /**
    * Calculate map coordinates from gps coordinates.
    */
   public int calculateX(float x){

      double nX = ((0-image.getWidth(this))/(nwLon - seLon))*( x - nwLon);

      return (int)(nX*map.zoomFactor);
   }
   /**
    * Set the stop set.
    */
   public void setStops(CbnStopSet sSet){
         this.stops = sSet;
   }
   /**
    * Set the bus data.
    */
   public void setBuses(CbnBuses buses){
      this.buses = buses;
   }
   /**
    * Timer Task to center the map, when it is first loaded.
    */
   class CenterTask extends TimerTask{
      String command = "Center";
      int k = 0;
      public void run()
      {
         if( k < 4){
            fireActionEvent(command);
            k++;
         }
         else
            cTimer.cancel();
      }
   }
   /**
    * Timer to receive the bus data from the 
    * database.
    */
   class GetBusesTask extends TimerTask{
      String command = "GetBuses";
      public void run(){
         fireActionEvent(command);
      }
   }  
   /**
    * Fire an action event.
    */
   void fireActionEvent (String command) {
      if(actionListener != null) {
         actionListener.actionPerformed (new ActionEvent(
	   this, ActionEvent.ACTION_PERFORMED, command));
      }
   }
   /**
    * Add the actionListener.
    */
  public void addActionListener (ActionListener listener) {
     actionListener = AWTEventMulticaster.add (actionListener, listener);
     
  }
  /**
  * Prints to the standard out, the memory statistics.
  * @param messageCall The message to be printed before printing the statistics to screen.
  */
  public static void printMemUsage(String messageCall)
  {
      System.out.println("--------------------------------------------------");
      System.out.println(messageCall);
      System.out.println("Max-Memory   : "+(Runtime.getRuntime().maxMemory()*(1.0/1024.0)*(1.0/1024.0))+" MB");
      System.out.println("Total-Memory : "+(Runtime.getRuntime().totalMemory()*(1.0/1024.0)*(1.0/1024.0))+" MB");
      System.out.println("Free-Memory  : "+(Runtime.getRuntime().freeMemory()*(1.0/1024.0)*(1.0/1024.0))+" MB");
      System.out.println("Memory-Usage : "+
	 ((Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory())*(1.0/1024.0)*(1.0/1024.0))+" MB");
      System.out.println("--------------------------------------------------");
  } // END: printMemUsage
}
