package bullybus;
import java.io.*;
import java.sql.*;
import java.util.*;
import java.lang.Math;
/**
 * This class is an vector of CbnRouteData," it can
 * be sent by a servlet.
 * 
 * @see CbnRouteData
 * @author Ron Lewis
 * @version 1.0 07/30/04
 */
public class CbnRouteDataSet implements Serializable{
    static final long serialVersionUID = 6798845900015401556L;
    
    private Vector data;
    /**
     * The constructor.
     */
    public CbnRouteDataSet(){
	data = new Vector();
    }
    /**
     * Append a CbnRouteData element to the vector.
     * @return false if error, true if added.
     */
    public boolean add(CbnRouteData D)
    {
	return data.add(D);
    }
    /**
     * Return the data element at the index i.
     * @return the element at the index i.
     */
    public CbnRouteData getData(int i)
    {
	return (CbnRouteData)data.elementAt(i);
    }
    /**
     * Returns the size of the vector.
     * @return the size of the vector.
     */
    public int getSize()
    {
	return data.size();
    }
    /**
     * Return the distance to the next stop.
     * @return the distance to the next stop.
     * @param gpsN the buses current gps latitude.
     * @param gpsW the buses current gps longitude.
     */
    public double distToNextStop(double gpsN, double gpsW)
    {
        double d = 1000;
        double temp = 0.0;
        CbnRouteData T;
	for(int i = 0; i < data.size(); i++)
	{   
            T = (CbnRouteData)data.elementAt(i);
	    temp = GpsCalculator.calculateDistance(gpsN, gpsW, T.gpsN, T.gpsW);
            if(temp <= d)
		d = temp;
	}
	    
	return d;
    }
    /*
     * Return the Estimated Time of Arrival to the given stop.
     * @return the Estimated Time of Arrival to the given stop.
     * @param stopName the specified stop.
     * @param gpsN the buses current gps latitude.
     * @param gpsW the buses current gps longitude.
     */
    public String etaToStop(String stopName, double gpsN, double gpsW)
    {

        double d = 1000;
        double temp = 0.0;
        CbnRouteData T;
	//Find the point we are closest to.
	int j = 0; //index of the closest point on route.
	for(int i = 0; i < data.size(); i++)
	{   
            T = this.getData(i);
	    temp = GpsCalculator.calculateDistance(gpsN, gpsW, T.gpsN, T.gpsW);
            if(temp < d)
	    {
		d = temp;
		j = i;
	    }
	}	
	//System.out.println(j);
	boolean stopNotFound = true;
	boolean doneLooking = false;
	d = this.getData(j).distToNextStop;
	String currentStop = this.getData(j).nextStop;
	// Check to see if we are at the stop.
	if(this.getData(j).nextStop.equals(stopName) && this.getData(j).isAStop)
	{
	    d = 0;
	    stopNotFound = false;
	}// And also check to see if the next stop is the one we are looking for.
	// and if so return the distance stored in the data element.
	else if(this.getData(j).nextStop.equals(stopName))
	{
	    d = this.getData(j).distToNextStop;
	    stopNotFound = false;
	} 
	else{
	    //Do Nothing
	}
	int k = 0;
	while(stopNotFound && !doneLooking) //Look for stop until we find it or look at all the elements
        {   
            k++;                           //increment k.            
            j = (j+1)%(data.size());   //increment j and make sure it does not exceed the size of the vector.
	    if(!currentStop.equals(this.getData(j).nextStop))
	    {
		currentStop = this.getData(j).nextStop;
		if( j == 0){
		    d += this.getData(data.size() - 1).distToNextStop;
		}
		else{
		    d+=this.getData(j -1).distToNextStop;
		}
		if(this.getData(j).nextStop.equals(stopName))
		{		 
		    stopNotFound = false;
		}
	    }
	    if( k < data.size()) //We have been through the list once
	    {
		doneLooking = false;
	    }
	    else{
	        doneLooking = true;
		
	    }	
	}
	int time = (int)((d/10)*60);
	//System.out.println("D is: "+d);
	//System.out.println("The time is: "+time+" min");
	if(stopNotFound && doneLooking)  // If the stop is not found the chances are that the stop is not on this route.
	{
	    return("N/A");  // So return n/a
	}
	else
	{
	    return(time+" min");  // The stop was found return the estimated time of arrival.
	}

    }    
}




