/*
 * @(#)GaugeGps.java        2.0 01/25/2005
 *
 * Copyright (c) 2004 CAVS, All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted.
 *
 */
package bullybus;
import java.awt.*;
import java.awt.image.*;
/** 
 * A class to display gps Coordinates to the
 * Gauge Panel. 
 * 
 *
 * @author Will Jenkins
 * @version 2.0 01/25/2005
 */
public class GaugeGPS extends Canvas {


    private float []currGPS = new float[2];
    private boolean draw3D = false;
    private String busID;
    /**
     * Constructor, Create a new GaugeGPS.
     * @param b the id of the bus in question.
     */
    public GaugeGPS(String b) {
    	busID = b;
	init();
    };  
    /**
     * The initialize function.
     */
    protected void init() {
	setForeground(new Color(137,32,52));
	setFont(new Font("Courier", Font.BOLD, 16));	
    };    
    /**
     * Set the current gps coordinates.
     * @param gpsN the latitude portion of the coordinate
     * @param gpsW the longitude portion of the coordinate
     */
    public void setCurrentGPS(float gpsN, float gpsW) {
	currGPS[0] = gpsN;
	currGPS[1] = gpsW;
	repaint();
    };
    /**
     * Returns the current gps coordinates.
     * @return the current gps coordinates.
     */    
    public float[] getCurrentGPS() {
	return currGPS;
    }
    /**
     * At update(); draw into an off-screen image and put up the whole
     * image.  This clears up any flickering.
     */
    public void update(Graphics g) {
  	Image osImg = createImage(getWidth(), getHeight());
	osImg.getGraphics().fillRect(0,0,getWidth(), getHeight());
	paint(osImg.getGraphics());
	g.drawImage(osImg,0,0,null);
	//paint(g);
    };
     /**
     * The set the size of the panel to the preferred size.
     */      
    public void setSize(int w, int h) {
	super.setSize(w, h);
	repaint();
    };
     /**
     * The paint function, paints the data to the screen.
     */   
    public void paint(Graphics g) {
	g.setColor(new Color(226,226,199));
	
	if (draw3D) {
	    for (int i=0;i<=3;i++) {
		g.fill3DRect(i,i,getWidth()-(2*i), getHeight()-(2*i),true);
	    }
	} else {
	    g.fillRect(0,0, getWidth(), getHeight());
	}
	
	g.setColor(getForeground());
	g.drawString(busID.toUpperCase(),
				 5,
				 getHeight() - getHeight()/4);
	if(currGPS[0] != 0)
	    g.drawString("GPS: " + currGPS[0] + "N " + currGPS[1] + "W",
			 (int)(getWidth()/2 - 110),
			 getHeight() - getHeight()/4);
	else
	    g.drawString("GPS: XX.XXXXN XX.XXXXW",
			 (int)(getWidth()/2 - 110),
			 getHeight() - getHeight()/4);
    };
    /**
     * The function to call the paint function..
     */   
    public void repaint() {
	super.repaint();
    };
}
