/*
 * @(#)RollOverImageButton.java  2.0 01/25/2005
 *
 * Copyright (c) 2004 CAVS, All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted.
 *
 */
package bullybus;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.awt.Canvas.*;

/** 
 * A class to display an image on a canvas.
 * This is a specialized version of ImageButton.
 * Adds an implementation of mouse overs.
 * @author Ron Lewis
 * @version 2.0 01/25/2005
 */
public class RollOverImageButton extends java.awt.Canvas
{
   /**
   * The actionListener events are fired to.
   */ 
   private ActionListener actionListener;
   /**
    * Accepting and processing mouse events
    */
   private boolean active;
   /**
    * Image to be displayed on mouse over.
    */
   protected Image on; 
   /**
    * Image to be displayed normally.
    */
   protected Image off;
   /**
    * Is this button selected, Displays the on image.
    */
   public boolean isOn;
   /**
    * Displays on image during mouse over event.
    */
   protected boolean mouseOver;
   /**
    * The width of the image
    */
   protected int width = -1;
   /**
    * The height of the image
    */
   protected int height = -1;
   /**
    * The actionCommand that is fired by this button accessed by
    * "e.getActionCommand()"
    */
   protected String command;
   /**
    * Constructs an image button with "off" as the primary image to be
    * displayed and on as the image to be displayed when the mouse is 
    * over the button. Command is the action command that is fired upon
    * an actionEvernt.
    */
   public RollOverImageButton(Image on, Image off, String command) 
   {
      this.on = on;
      this.off = off;;
      isOn = false;
      setSize();
      active = true;
      this.command = command;
   }
   /**
    * Constructs an image button with "off" as the primary image to be
    * displayed and on as the image to be displayed when the mouse is 
    * over the button. Command is the action command that is fired upon
    * an actionEvernt is set to null.
    */
   public RollOverImageButton (Image on, Image off) 
   {
      this.on = on;
      this.off = off;
      setSize();
      active = true;
      this.command = null;
   }   
   /**
    * Constructs an image button by createing the images from the 
    * ImageProducers.  The ipOn parram is used to create the image
    * that will be displayed on mouse over.  The "ipOff" parram will be
    * used to create the image that will be display normally. Command is 
    * the action command that is fired upon an actionEvernt.
    */
   public RollOverImageButton (ImageProducer ipOn, ImageProducer ipOff, String command) 
   {
      this.on = createImage(ipOn);
      this.off = createImage(ipOff);
      setSize();
      active = true;
      this.command = command;
   }
   /**
    * Sets the canvas size to the size of the images.  Called by
    * the constructor and imageUpdate.
    */
   protected synchronized void setSize () 
   {
      if (width < 0) 
         width = off.getWidth(this);
      if (height < 0) 
	 height = off.getHeight(this);
      if (width > 0 && height > 0) 
         setSize((int)(width),(int)(height));
   }
   /**
    * Called to resize the window container.
    * Not to be called by user code.
    */
   public synchronized boolean imageUpdate (Image img, int infoflags, int x, int y, int w, int h) 
   {
      if (this.width < 0 || this.height < 0) 
      {
         if ((infoflags & ImageObserver.WIDTH) != 0) 
	    this.width = w;
	 if ((infoflags & ImageObserver.HEIGHT) != 0) 
	    this.height = h;
	 if (this.width > 0 && this.height > 0) 
	 {
	    setSize((int)(this.width) , (int)(this.height));
	    Component c = this;
	    while ((c = c.getParent()) != null && ! (c instanceof Window));
	    if (c != null) 
	       ((Window)c).pack();
	 }
      }
      return super.imageUpdate(img, infoflags, x, y, width, height);
   }
   /**
    * The main paint routine.
    */
   public void paint (Graphics g) 
   {
       Dimension d = getSize();
		
       if (width > 0 && height > 0 && d.width > 0 && d.height > 0) 
       {
          float hAspect = (float)d.height/height, wAspect = (float)d.width/width;
	  int h, w;
	  if (hAspect < wAspect) 
	  { 
	     h = (int)(off.getHeight(this)); 
	     w = (int)(off.getWidth(this)); 
	  }
	  else 
	  { 
	     w = (int)(off.getHeight(this));
	     h = (int)(off.getWidth(this));
	  }
	  if(isOn || mouseOver)
	     g.drawImage(on, 0, 0,h,w,Color.white, this);
	  else
	     g.drawImage(off, 0,0,h,w,Color.white, this);
       }
				
   }
   /**
    * Returns the width of the button.
    * @return int
    */
   public int getImageWidth()
   {
      return this.width;
   }
   /**
    * Returns the height of the button.
    * @return int
    */
   public int getImageHeight()
   {
      return this.height;
   }
   /**
    * Adds an actionlistener. Activates processing of
    * mouse events.
    */
   public void addActionListener (ActionListener listener) {
      actionListener = AWTEventMulticaster.add (actionListener, listener);
      enableEvents(AWTEvent.MOUSE_EVENT_MASK);
   } 
    
   public void removeActionListener (ActionListener listener) {
       actionListener = AWTEventMulticaster.remove (actionListener, listener);
   }
    /**
    * Sets the "active" parameter. If the cell is active then
    * it responds to mouse events.
    * 
    * @param active new "active" state value
    */
    void setActive (boolean active) {
       this.active = active;
    }
    /**
     * Resonds to MouseEvents and Fires actionEvents to the actionListener.
     * Enabled when active = true.
     */
    public void processMouseEvent (MouseEvent e) {

       if (active) {
        
          switch (e.getID()) {
	  
	  case MouseEvent.MOUSE_ENTERED:
	   if(!mouseOver)
	   {
	       mouseOver = true;
	       repaint();
	   }
	   Cursor cursor = new Cursor(Cursor.HAND_CURSOR );
	   setCursor(cursor);
	   break;
	  case MouseEvent.MOUSE_EXITED:
	   if(mouseOver)
	   {
	       mouseOver = false;
	       repaint();
	   }
	   Cursor cursorD = new Cursor(Cursor.DEFAULT_CURSOR);
	   setCursor(cursorD);	      
	   break;
          case MouseEvent.MOUSE_PRESSED:
	  // Note: under UNIX for an unknown reason (AWT bug?) modifs is 0
	  // when the left button is pressed, but 16 when it is released!
	  // This is not the case under Win95.
	     break;
          case MouseEvent.MOUSE_RELEASED:
	     break;
          case MouseEvent.MOUSE_CLICKED:
	  // if right button is clicked
	      fireActionEvent ();
	  // if right button is clicked when in state MARKED
	
	    break; 
	}
      }
    super.processMouseEvent(e);
   }
  
  /**
   * Called to Fire an ActionEvent to the registered ActionListener
   *
   * @param command a String denoting the type of action (defaults to label, 
   * ).
   */
  void fireActionEvent () {
    if(actionListener != null) {
      actionListener.actionPerformed (new ActionEvent(
	 this, ActionEvent.ACTION_PERFORMED, command));
    }
  }

}
